@@ -1,9 +1,10 @@
 from __future__ import absolute_import, division, print_function
-from .convert.api import open_raw
-from .echodata.api import open_converted
-from .process import Process
-from . import calibrate
 
 from _echopype_version import version as __version__  # noqa
 
-__all__ = [open_raw, open_converted, calibrate]
+from . import calibrate
+from .convert.api import open_raw
+from .echodata.api import open_converted
+from .process import Process  # noqa
+
+__all__ = ["open_raw", "open_converted", "calibrate"]
